/*
    Project: TCGINA
    Version: V1.15
    File   : tcgkeyfile.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tcgpriv.h"
#include "pwdlg.h"
#include "mediumdlg.h"

#ifdef SUPPORT_KEYFILES

#define KF_MAX_ITEMS 200

static BOOL TCG_DecryptKeyFile(PTCG_KEY_DATA ptkd, PTCG_SETTINGS pts,
  PKFI_DATA pkdEncryptedData, DWORD cItems)
{
    size_t    cchPassword;
    PKFI_DATA pkdEncryptedDataEnd;

    TC_Assert(ptkd);
    TC_Assert(pts);
    TC_Assert(pkdEncryptedData);

    cchPassword = strlen(ptkd->aszPassword);

    pkdEncryptedDataEnd = pkdEncryptedData + cItems;
    for (; pkdEncryptedData != pkdEncryptedDataEnd; pkdEncryptedData++) {
        if (cchPassword) {
            if (KFI_InitEncryptionData(pkdEncryptedData,
              &ptkd->tmp.ked, "", 0) == KFIE_OK) {
                if (KFI_Decrypt(&ptkd->kdDecryptedData, pkdEncryptedData,
                  &ptkd->tmp.ked) == KFIE_OK) {
                    goto decrypted;
                }
            }
        }
        if (KFI_InitEncryptionData(pkdEncryptedData, &ptkd->tmp.ked,
          ptkd->aszPassword, cchPassword) != KFIE_OK) {
            continue;
        }
        if (KFI_Decrypt(&ptkd->kdDecryptedData, pkdEncryptedData,
          &ptkd->tmp.ked) != KFIE_OK) {
            continue;
        }
decrypted:
        if (pts->dwKFID) {
            if (pts->dwKFID != ptkd->kdDecryptedData.dwID) {
                continue;
            }
        }

        return TRUE;
    }

    return FALSE;
}

typedef struct
{
    LPTSTR  pszFileName;
    LPCTSTR pszDrives;
} FILE_EXISTS_DATA, *PFILE_EXISTS_DATA;

static BOOL CALLBACK TCG_KeyFileExists(VOID *pCtx)
{
    LPTSTR pszFileName = ((PFILE_EXISTS_DATA)pCtx)->pszFileName;
    LPCTSTR pszDrives = ((PFILE_EXISTS_DATA)pCtx)->pszDrives;

    if (*pszDrives == TEXT('\0')) {
        return TC_FileExists(pszFileName);
    }

    while (*pszDrives) {
        *pszFileName = *pszDrives;
        if (TC_FileExists(pszFileName)) {
            return TRUE;
        }
        pszDrives++;
    }

    return FALSE;
}

BOOL WINAPI TCG_ReadKeyFile(PTCG_CONTEXT pCtx, PTCG_KEY_DATA ptkd, PTCG_SETTINGS pts)
{
    FILE_EXISTS_DATA fed;
    TCHAR            szDrives[26+1+1];
    PKFI_DATA        pkd;

    TC_Assert(ptkd);
    TC_Assert(pts);

    if (*pts->szKFFileName == TEXT('\0')) {
        *ptkd->aszPassword = '\0';
        memset(&ptkd->kdDecryptedData, '\0', sizeof(KFI_DATA));
        return TRUE;
    }

    fed.pszDrives = szDrives;
    fed.pszFileName = pts->szKFFileName;
    if (pts->szKFFileName[1] == TEXT(':')) {
        szDrives[0] = pts->szKFFileName[0];
        if (!TC_CopyString(szDrives + 1, 26+1, pts->szKFDrives)) {
            *szDrives = TEXT('\0');
        }
    } else {
        *szDrives = TEXT('\0');
    }

    pkd = (PKFI_DATA) TC_Alloc(KF_MAX_ITEMS*sizeof(KFI_DATA));
    if (pkd == NULL) {
        *ptkd->aszPassword = '\0';
        memset(&ptkd->kdDecryptedData, '\0', sizeof(KFI_DATA));
        return FALSE;
    }

    for (;;) {
        HANDLE hFile;
        DWORD  cItems;

        if (!TCG_KeyFileExists(&fed)) {
            MEDIUM_DIALOG_DATA mdd;
            memset(&mdd, '\0', sizeof(mdd));
            mdd.pszCaption = TEXT("Password File");
            mdd.pszText = TEXT("Insert password medium!");
            mdd.dwTimeout = pts->dwKFMediumTimeout;
#ifdef SUPPORT_AUTOMOUNT
            if (!pCtx->fAutoMountDone) {
                if (mdd.dwTimeout == 0) {
                    mdd.dwTimeout = TCG_AUTOMOUNT_DEFAULT_TIMEOUT;
                }
            }
#endif
            mdd.pfMediumAvailable = TCG_KeyFileExists;
            mdd.pContext = &fed;
            if (pCtx->pWlxFuncs->WlxDialogBoxParam(pCtx->hWlx,
              g_hinstDLL, MAKEINTRESOURCE(IDD_MEDIUM), NULL,
              MediumDialogProc, (LPARAM)&mdd) != IDOK) {
                memset(&ptkd->kdDecryptedData, '\0', sizeof(KFI_DATA));
                TC_Free(pkd);
                return FALSE;
            }
        }

        hFile = CreateFile(pts->szKFFileName, GENERIC_READ, FILE_SHARE_READ,
          NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hFile == INVALID_HANDLE_VALUE) {
            continue;
        }
        if (!SetFilePointerEx(hFile, pts->liKFFileOffset, NULL,
          FILE_BEGIN)) {
            CloseHandle(hFile);
            continue;
        }
        if (!ReadFile(hFile, pkd, KF_MAX_ITEMS * sizeof(KFI_DATA),
          &cItems, NULL)) {
            cItems = 0;
        }
        CloseHandle(hFile);
        cItems /= sizeof(KFI_DATA);
        if (cItems == 0) continue;

        if (*pts->szKFPassword) {
            TC_CopyStringToUTF8(ptkd->aszPassword,
              sizeof(ptkd->aszPassword), pts->szKFPassword);
        }
        if (TCG_DecryptKeyFile(ptkd, pts, pkd, cItems)) {
            break;
        }

        if (*pts->szKFPassword == TEXT('\0') &&
          !pts->dwKFNoPasswordDialog) {
            PASSWORD_DIALOG_DATA pdd;
            memset(&pdd, '\0', sizeof(pdd));
            if (pts->dwTCOptions & TC_DISPLAY_PASSWORD) {
                pdd.fDisplayPassword = TRUE;
            }
            pdd.pszCaption = TEXT("Enter Password For Password File");
            pdd.uCodePage = CP_UTF8;
            pdd.pPassword = ptkd->aszPassword;
            pdd.cbPasswordSize = KFI_MAX_PASSWORD;
            pdd.pwszTempPassword = ptkd->tmp.wszTempPassword;
            pdd.dwMinPasswordLength = 1;
            pdd.dwMaxPasswordLength = KFI_MAX_PASSWORD_LEN;
            pdd.dwTimeout = pts->dwKFPasswordTimeout;
            pdd.dwTotalTimeout = pts->dwKFPasswordTotalTimeout;
#ifdef SUPPORT_AUTOMOUNT
            if (!pCtx->fAutoMountDone) {
                if (pdd.dwTotalTimeout == 0) {
                    pdd.dwTotalTimeout = TCG_AUTOMOUNT_DEFAULT_TIMEOUT;
                }
            }
#endif
            if (pCtx->pWlxFuncs->WlxDialogBoxParam(pCtx->hWlx,
              g_hinstDLL, MAKEINTRESOURCE(IDD_PASSWORD), NULL,
              PasswordDialogProc, (LPARAM)&pdd) == IDOK) {
                if (TCG_DecryptKeyFile(ptkd, pts, pkd, cItems)) {
                    break;
                }
            } else {
                memset(&ptkd->kdDecryptedData, '\0', sizeof(KFI_DATA));
                TC_Free(pkd);
                return FALSE;
            }
        }
    }
    TC_Free(pkd);

    *ptkd->aszPassword = '\0';

    if (ptkd->kdDecryptedData.dwMountMask) {
        DWORD dwMask = ptkd->kdDecryptedData.dwMountMask & 0x1f;
        pts->dwTCMountOptions &= ~dwMask;
        pts->dwTCMountOptions |= (ptkd->kdDecryptedData.dwMountMask & dwMask);
    }

    return TRUE;
}

#endif